#!/usr/bin/wish

#############################################################################
# wd2ctd - deck converter for WinDeck
# Copyright (C) 1998  Chris Gregg
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#############################################################################
#################################
# GLOBAL VARIABLES
#
global widget; 
global data; 
global data1;

#################################
# USER DEFINED PROCEDURES
#
proc init {argc argv} {

}

init $argc $argv


proc main {argc argv} {
    readData "."
    write_text "\n*** Done reading datafiles\n"
}

proc readData {wd2dir} {
    global data
    global data1

    set datFile [glob -nocomplain *.dat]

    if {! [llength $datFile] } {
        write_text "*** No datafiles were read.\n"
        write_text "Did you copy wd2ctd.tcl into the WinDeck data directory?.\n"
        write_text "wd2ctd.tcl should be run from the same directory that the data files (anh.dat, premiere.dat, etc.) are found.\n"
        write_text "Quit this program, copy it to that directory and try again.\n"
        return
    }


    foreach {file} $datFile {
        write_text "*** Reading ${wd2dir}/${file}\n"
        if {[file exists ${wd2dir}/${file}]} {
        write_text "\["

            set fp [open ${wd2dir}/${file} r]
            set i 0;
            while {! [eof $fp] } {
                set dataline [gets $fp]
                if {[regexp {^[ld];} $dataline]} {
                    if {$file == "cards.dat"} {
                        lappend data1 $dataline
                    } else {
                        set datlist [split $dataline ";"]
			set setlist [split [lindex $datlist 3] {}]
			for { set i 0 } { $i < [llength $setlist] } { incr i } {
				lappend data "[join [lrange $datlist 0 2] ";"];[lindex $setlist $i];[join [lrange $datlist 4 end] ";"]" #$dataline
			}
                    }

                    if {${i}%10 == 0} {
                        write_text "."
                    }
                    incr i
                }
            }
            write_text "\]\n"
            close $fp
        } else {
            write_text "*** WD data ${wd2dir}/${file} does not exist.\n"
        }
    }
    lappend data {}
    lappend data1 {}
}

proc convertFile {} {
    set itypes {
        {{Windeck1 Decks} {.wdi} }
        {{Windeck2 Decks} {.deck} }
        {{All Files}       *      }
    }
    set otypes {
        {{CardTable Decks} {.ctd} }
        {{All Files}       *      }
    }
    set file [tk_getOpenFile -defaultextension "deck" -filetypes $itypes -parent .top17 -title "Open Deck"]
    if {$file != ""} {
        if {[regexp -nocase "(.*)\\.deck" $file matchStr deckName]} {
            set ofile "${deckName}.ctd"
        } else {
            set ofile "cardtable.ctd"
        }
        set ofile [tk_getSaveFile -defaultextension "ctd" -initialfile $ofile -filetypes $otypes -parent .top17 -title "Save Deck As"]
        if {$ofile != ""} {
            wdConvert $file $ofile
        }
    }
}

proc write_text {string} {
#
    .top17.cpd22.03 configure -state normal
    .top17.cpd22.03 insert end "$string"
    .top17.cpd22.03 configure -state disabled
    .top17.cpd22.03 see end
    update idletasks
    return
}

proc escapeChars {str} {
    regsub -all {\(} $str {\(} str
    regsub -all {\)} $str {\)} str
    regsub -all {\[} $str {\[} str
    regsub -all {\]} $str {\]} str
    regsub -all {\$} $str {\$} str
    regsub -all {\.} $str {\.} str
    regsub -all {\*} $str {\*} str
    regsub -all {\^} $str {\^} str
    regsub -all {\?} $str {\?} str
    return $str
}

proc wd1ctd {fp ofp} {
    global data1
    set backImg "blotch.gif"

    set data $data1

    gets $fp hidden
    gets $fp deckData

    regsub -all -- {;} $deckData " " deckData
    regsub -all -- {-} $deckData " " deckData
    
    set length [llength $data]

    foreach {index count} $deckData {
    puts "$index x $count"

        if {$index > $length} {
            write_text "*** $index is out of range!\n"
        } else {
            set card [lindex $data $index]

            regexp {^.;.;.;.;[^;]*;([^;]*);.*} $card matchStr name

            write_text "$count x \"$name\"\n"

            if {[regexp {^d;} $card]} {
                set cardBack "imp.gif"
            } else {
                set cardBack "reb.gif"
            }
            if {$cardBack != $backImg} {
                puts $ofp "back $cardBack"
                set backImg $cardBack
            }

            puts $ofp "[semiToCard $card] $count"
        }
    }

    close $fp
    close $ofp
    write_text "\n*** Done converting.\n"
}

proc wd2ctd {fp ofp} {
    global data
    set backImg "blotch.gif"

    while {! [eof $fp] } {
        set card [gets $fp]
        set reStr {[^;]*}
        set reCard "^($reStr);($reStr);($reStr);($reStr);($reStr)"
        if [regexp $reCard $card matchstr cardName cardSide cardSet cardCount cardVisible] {
            if {!$cardCount} {break}
            write_text "$cardCount x \"$cardName\"\n"
            # set index [lsearch $data "$cardSide;?;?;$cardSet;$reStr;$cardName;*"]
            set index [lsearch -regexp $data "$cardSide;.;.;$cardSet;$reStr;[escapeChars $cardName];.*"]
            if {$index == -1} {
                write_text "* $cardName not found *\n"
            } else {

                if {$cardSide == "d"} {
                    set cardBack "imp.gif"
                } else {
                    set cardBack "reb.gif"
                }

                if {$cardBack != $backImg} {
                    puts $ofp "back $cardBack"
                    set backImg $cardBack
                }

                puts $ofp "[semiToCard [lindex $data $index]] $cardCount"
            }
        }
    }
    close $fp
    close $ofp
    write_text "\n*** Done converting.\n"
}

proc wdConvert {infile outfile} {
    global data1
    global data
    set fp [open $infile r]
    set ofp [open $outfile w]

    write_text "\n*** Converting...\n"
    gets $fp type
    if {[regexp {^cards\.dat} $type]} {
        write_text "*** This appears to be a WinDeck 1 file...\n"
        if {[llength $data1] == 1} {
            write_text "*** cards.dat wasn't read in properly. Bailing.\n"
        } else {
            wd1ctd $fp $ofp
        }
    } elseif {[regexp {NEEDFILE} $type] || [regexp {VERSION} $type]} {
        write_text "*** This appears to be a WinDeck 2 file...\n"
        if {[llength $data] == 1} {
            write_text "*** WinDeck 2 data files not read in properly. Bailing.\n"
        } else {
            wd2ctd $fp $ofp
        }
    } else {
    }
}

proc semiToCard {raw_card} {

  set reStr {[^;]*}
  set reForce {[ld]}

  set reCharacter "^($reForce);(c);(.);(.);($reStr);($reStr);($reStr);($reStr);($reStr);($reStr);($reStr);($reStr);($reStr)"
  set reCreature  "^($reForce);(r);(.);(.);($reStr);($reStr);($reStr);($reStr);($reStr);($reStr);($reStr);($reStr);($reStr)"
  set reDevice    "^($reForce);(d);(.);(.);($reStr);($reStr);($reStr);($reStr);($reStr);($reStr);($reStr);($reStr)"
  set reEffect    "^($reForce);(e);(.);(.);($reStr);($reStr);($reStr);($reStr);($reStr);($reStr)"
  set reEpicEvent "^($reForce);(E);(.);(.);($reStr);($reStr);($reStr);($reStr);($reStr);($reStr)"
  set reObjective "^($reForce);(O);(.);(.);($reStr);($reStr);($reStr);($reStr);($reStr);($reStr)"
  set reInterrupt "^($reForce);(i);(.);(.);($reStr);($reStr);($reStr);($reStr);($reStr);($reStr)"
  set reJediTest  "^($reForce);(j);(.);(.);($reStr);($reStr);($reStr);($reStr);($reStr);($reStr)"
  set reLocation  "^($reForce);(l);(.);(.);($reStr);($reStr);($reStr);($reStr);($reStr);($reStr);($reStr);($reStr);($reStr)"
  set reStarship  "^($reForce);(s);(.);(.);($reStr);($reStr);($reStr);($reStr);($reStr);($reStr);($reStr);($reStr);($reStr)"
  set reVehicle   "^($reForce);(v);(.);(.);($reStr);($reStr);($reStr);($reStr);($reStr);($reStr);($reStr);($reStr);($reStr)"
  set reWeapon    "^($reForce);(w);(.);(.);($reStr);($reStr);($reStr);($reStr);($reStr);($reStr);($reStr);($reStr)"
#                   force     type sub set uniq     name     rarity   destiny  stats    icons    lore     text

  set c_class ""
  set c_destiny ""
  set c_dicons ""
  set c_icons ""
  set c_licons ""
  set c_lore ""
  set c_name ""
  set c_rarity ""
  set c_set ""
  set c_stats ""
  set c_subtype ""
  set c_text ""
  set c_type ""
  set c_uniq ""
  set invalid 0

  if [regexp $reLocation $raw_card matchstr c_alignment c_type c_subtype \
    c_set c_uniq c_name c_rarity c_location c_icons c_licons c_ltxt \
    c_dicons c_dtxt] then {
    set c_text "$c_ltxt#$c_dtxt"
    set c_class $c_location

    return [showLocation $c_alignment $c_type $c_subtype \
    $c_set $c_uniq $c_name $c_rarity $c_location $c_icons $c_licons $c_ltxt \
    $c_dicons $c_dtxt]

  } \
  elseif [regexp $reCharacter $raw_card matchstr c_alignment c_type c_subtype \
    c_set c_uniq c_name c_rarity c_destiny c_stats c_class c_icons \
    c_lore c_text] then {
        return [showCharacter $c_alignment $c_type $c_subtype $c_set \
                              $c_uniq $c_name $c_rarity $c_destiny \
                              $c_stats $c_class $c_icons $c_lore $c_text ]
  } \
  elseif [regexp $reInterrupt $raw_card matchstr c_alignment c_type c_subtype \
    c_set c_uniq c_name c_rarity c_destiny c_lore c_text] then {

        return [showInterrupt $c_alignment $c_type $c_subtype $c_set $c_uniq \
                              $c_name $c_rarity $c_destiny $c_lore $c_text]
        
  } \
  elseif [regexp $reEffect $raw_card matchstr c_alignment c_type c_subtype \
    c_set c_uniq c_name c_rarity c_destiny c_lore c_text] then {
        return [showEffect $c_alignment $c_type $c_subtype $c_set $c_uniq \
                           $c_name $c_rarity $c_destiny $c_lore $c_text]
  } \
  elseif [regexp $reEpicEvent $raw_card matchstr c_alignment c_type c_subtype \
    c_set c_uniq c_name c_rarity c_destiny c_lore c_text] then {
        return [showEpicEvent $c_alignment $c_type $c_subtype $c_set $c_uniq \
                              $c_name $c_rarity $c_destiny $c_lore $c_text]
  } \
  elseif [regexp $reObjective $raw_card matchstr c_alignment c_type c_subtype \
    c_set c_uniq c_name c_rarity c_destiny c_text1 c_text2] then {
        return [showObjective $c_alignment $c_type $c_subtype $c_set $c_uniq \
                              $c_name $c_rarity $c_destiny $c_text1 $c_text2]
  } \
  elseif [regexp $reJediTest $raw_card matchstr c_alignment c_type c_subtype \
    c_set c_uniq c_name c_rarity c_destiny c_lore c_text] then {
        return [showJediTest $c_alignment $c_type $c_subtype $c_set $c_uniq \
                             $c_name $c_rarity $c_destiny $c_lore $c_text]
  } \
  elseif [regexp $reStarship $raw_card matchstr c_alignment c_type c_subtype \
    c_set c_uniq c_name c_rarity c_destiny c_stats c_class c_icons \
    c_lore c_text] then {
        return [showStarship $c_alignment $c_type $c_subtype $c_set $c_uniq \
                             $c_name $c_rarity $c_destiny $c_stats $c_class \
                             $c_icons $c_lore $c_text]
  } \
  elseif [regexp $reVehicle $raw_card matchstr c_alignment c_type c_subtype \
    c_set c_uniq c_name c_rarity c_destiny c_stats c_class c_icons \
    c_lore c_text] then {
        return [showVehicle $c_alignment $c_type $c_subtype $c_set $c_uniq \
                            $c_name $c_rarity $c_destiny $c_stats $c_class \
                            $c_icons $c_lore $c_text]
  } \
  elseif [regexp $reWeapon $raw_card matchstr c_alignment c_type c_subtype \
    c_set c_uniq c_name c_rarity c_destiny c_stats c_icons \
    c_lore c_text] then {
        return [showWeapon $c_alignment $c_type $c_subtype \
    $c_set $c_uniq $c_name $c_rarity $c_destiny $c_stats $c_icons \
    $c_lore $c_text]
  } \
  elseif [regexp $reDevice $raw_card matchstr c_alignment c_type c_subtype \
    c_set c_uniq c_name c_rarity c_destiny c_stats c_icons \
    c_lore c_text] then {
        return [showDevice $c_alignment $c_type $c_subtype \
    $c_set $c_uniq $c_name $c_rarity $c_destiny $c_stats $c_icons \
    $c_lore $c_text]
  } \
  elseif [regexp $reCreature $raw_card matchstr c_alignment c_type c_subtype \
    c_set c_uniq c_name c_rarity c_destiny c_stats c_class c_icons \
    c_lore c_text] then {
        return [showCreature $c_alignment $c_type $c_subtype $c_set \
                             $c_uniq $c_name $c_rarity $c_destiny \
                             $c_stats $c_class $c_icons $c_lore $c_text ]
  } else {
    set invalid 1
  }
}

proc showInterrupt {alignment type subtype set uniq name rarity destiny \
                    lore text} {

    set string "card "
    append string [makeTitle $name $destiny "" ""]

    append string "\"$uniq$name ($destiny)\\n"
    append string "[showSide $alignment] [showInterSub $subtype] \[$rarity\]\\n"
    append string "Set: [showSet $set]\\n"
    append string "Lore: [eq $lore]\\n"
    append string "Text: [eq $text]\\n\" "

    append string "#C00"
    return $string
}
proc showEffect {alignment type subtype set uniq name rarity destiny \
                    lore text} {

    set string "card "
    append string [makeTitle $name $destiny "" ""]

    append string "\"$uniq$name ($destiny)\\n"
    append string "[showSide $alignment] [showEffectSub $subtype] \[$rarity\]\\n"
    append string "Set: [showSet $set]\\n"
    append string "Lore: [eq $lore]\\n"
    append string "Text: [eq $text]\\n\" "

    append string "#C00"
    return $string
}

proc showEpicEvent {alignment type subtype set uniq name rarity destiny \
                    lore text} {

    set string "card "
    append string [makeTitle $name $destiny "" ""]

    append string "\"$uniq$name ($destiny)\\n"
    append string "[showSide $alignment] Epic Event \[$rarity\]\\n"
    append string "Set: [showSet $set]\\n"
    append string "Text: [eq $text]\\n\" "

    append string "#C0C"
    return $string
}

proc showObjective {alignment type subtype set uniq name rarity destiny \
                    text1 text2} {

    set string "card "
    append string [makeTitle $name $destiny "" ""]

    append string "\"$uniq$name ($destiny)\\n"
    append string "[showSide $alignment] Epic Event \[$rarity\]\\n"
    append string "Set: [showSet $set]\\n"
    append string "Text 1: [eq $text1]\\n"
    append string "Text 2: [eq $text2]\\n\" "

    append string "#C0C"
    return $string
}

proc showJediTest {alignment type subtype set uniq name rarity destiny \
                    lore text} {

    set string "card "
    append string [makeTitle $name $destiny "" ""]

    append string "\"$uniq$name ($destiny)\\n"
    append string "[showSide $alignment] Jedi Test #$subtype \[$rarity\]\\n"
    append string "Set: [showSet $set]\\n"
    append string "Text: [eq $text]\\n\" "

    append string "#440"
    return $string
}
proc showIcons {icons} {
    set string " "
    if {[string match "*Pilot*" $icons]} {
        # Yellow
        append string {\016}
    }
    if {[string match "*Astromech*" $icons]} {
        # Red
        append string {\014}
    }
    if {[string match "*Warrior*" $icons]} {
        # Blue
        append string {\018}
    }
    if {[string match "*Scomp*" $icons]} {
        # Grey
        append string {\012}
    }
    if {[string match "*Selective*" $icons]} {
        # Green
        append string {\017}
    }
    return $string

}
proc makeTitle {name destiny stats icons} {
    append title "[eq $name]"
    set destiny " ($destiny)"
    if {(9-[string length $title]%9) < [string length $destiny]} {
        append title [makePad $name]
    }
    append title "$destiny $stats"
    append title [showIcons $icons]
    return "\"$title\" "
}

proc showCharacter { alignment type subtype set uniq name rarity destiny \
                     stats class icons lore text } {
    set cardName [eq $name]
    set string "card "

    append string [makeTitle $name $destiny $stats $icons]

    append string "\"[eq $uniq$name] ($destiny)\\n"
    append string "[showSide $alignment] [showCharSub $subtype] \[$rarity\]\\n"
    append string "Set: [showSet $set]\\n"
    append string "$stats $class\\n"
    append string "Icons: [eq $icons]\\n"
    append string "Lore: [eq $lore]\\n"
    append string "Text: [eq $text]\\n\" "

    append string "#777"
    return $string
}
proc showCreature { alignment type subtype set uniq name rarity destiny \
                     stats class icons lore text } {
    set cardName [eq $name]
    set string "card "

    append string [makeTitle $name $destiny $stats $icons]

    append string "\"[eq $uniq$name] ($destiny)\\n"
    append string "[showSide $alignment] $class \[$rarity\]\\n"
    append string "Set: [showSet $set]\\n"
    append string "$stats\\n"
    append string "Icons: [eq $icons]\\n"
    append string "Lore: [eq $lore]\\n"
    append string "Text: [eq $text]\\n\" "

    append string "#CCC"
    return $string
}
proc showStarship { alignment type subtype set uniq name rarity destiny \
                    stats class icons lore text } {
    set string "card "
    append string [makeTitle $name $destiny $stats $icons]

    append string "\"[eq $uniq$name] ($destiny)\\n"
    append string "[showSide $alignment] [showStarSub $subtype]: $class \[$rarity\]\\n"
    append string "Set: [showSet $set]\\n"
    append string "$stats\\n"
    append string "Icons: [eq $icons]\\n"
    append string "Lore: [eq $lore]\\n"
    append string "Text: [eq $text]\\n\" "

    append string "#00C"
    return $string
}
proc showWeapon {alignment type subtype set uniq name rarity destiny stats \
                 icons lore text } {
    set string "card "
    append string [makeTitle $name $destiny $stats $icons]

    append string "\"[eq $uniq$name] ($destiny)\\n"
    append string "[showSide $alignment] [showWeapSub $subtype] \[$rarity\]\\n"
    append string "Set: [showSet $set]\\n"
    append string "$stats\\n"
    append string "Lore: [eq $lore]\\n"
    append string "Text: [eq $text]\\n\" "  

    append string "#0C0"
    return $string
}
proc showDevice {alignment type subtype set uniq name rarity destiny stats \
                 icons lore text } {
    set string "card "
    append string [makeTitle $name $destiny $stats $icons]

    append string "\"[eq $uniq$name] ($destiny)\\n"
    append string "[showSide $alignment] Device \[$rarity\]\\n"
    append string "Set: [showSet $set]\\n"
    append string "Lore: [eq $lore]\\n"
    append string "Text: [eq $text]\\n\" "  

    append string "#0C0"
    return $string
}
proc showVehicle { alignment type subtype set uniq name rarity destiny \
                    stats class icons lore text } {
    set string "card "
    append string [makeTitle $name $destiny $stats $icons]

    append string "\"[eq $uniq$name] ($destiny)\\n"
    append string "[showSide $alignment] [showVehiSub $subtype]$class \[$rarity\]\\n"
    append string "Set: [showSet $set]\\n"
    append string "$stats\\n"
    append string "Icons: [eq $icons]\\n"
    append string "Lore: [eq $lore]\\n"
    append string "Text: [eq $text]\\n\" "

    append string "#00C"
    return $string
}
proc showLocation { alignment type subtype set uniq name rarity location icons licons ltxt dicons dtxt} {

    set cardName [eq $name]

    set string "card \"$cardName"

    append string "[makePad $cardName]"

    for {set i 0} {$i < 3} {incr i} {
        if {$i < $dicons} {
            # red force
            append string {\009}
        } else {
            # blank
            append string "_"
        }
    }
    for {set i 0} {$i < 3} {incr i} {
        if {$i < $licons} {
            # blue force
            append string {\008}
        } else {
            # blank
            append string "_"
        }
    }
    append string "\" "

    append string "\"[eq $uniq$name]\\n"
    append string "[showSide $alignment] [showLocSub $subtype] \[$rarity\]\\n"
    append string "Set: [showSet $set]\\n"
    if {[string length $location]} {
        append string "$location\\n"
    }
    append string "Icons: $icons\\n"
    append string "Light \[$licons\]: [eq $ltxt]\\n"
    append string "Dark \[$dicons\]: [eq $dtxt]\" "

    append string "#000"
    return $string
}

proc makePad {str} {
    return "         "
    set string ""
    set padLen [expr 8 - (([string length $str])%9) ]
    for {set i 0} {$i < $padLen} {incr i} {
        append string "_"
    }
    return $string
}

proc eq0 {str} {
    
    if {[string index $str 0] == "\""} {
        set str "\\$str"
    }
    while {[regexp {(.*)([^\\])(".*)} $str matchVar s1 c s2]} {
        set str "${s1}${c}\\${s2}"
    }
    return $str
}

proc eq {str} {
    regsub -all {"} $str {\"} str
    regsub -all {\|} $str {;} str
    return $str
}

proc showWeapSub {type} {
        switch $type {
          c { return "Character Weapon" }
          a { return "Automated Weapon" }
          s { return "Starship Weapon" }
          v { return "Vehicle Weapon" }
          r { return "Artillery Weapon" }
          d { return "Deathstar Weapon" }
        }
}
proc showStarSub {type} {
    switch $type {
        c { return "Capitol" }
        f { return "Starfighter" }
        s { return "Squadron" }
    }
    return ""
}
proc showVehiSub {type} {
        switch $type {
          c { return "Combat Vehicle: " }
          a { append card "Creature Vehicle" }
          t { append card "Transport Vehicle" }
          s { append card "Shuttle Vehicle" }
        }
        return ""
}

proc showSet {set} {
    switch $set {
        p { return "Premiere" }
        U { return "Premiere Unlimited" }
        i { return "Premiere Introductory Two Player Game" }
        h { return "Hoth" }
        a { return "A New Hope" }
        E { return "Empire Strikes Back Introductory Two Player Game" }
        r { return "Rebel Leader" }
        j { return "Jedi Pack" }
        d { return "Dagobah" }
        c { return "Cloud City" }
        s { return "Special Edition" }
        J { return "Jabba's Palace" }
        S { return "Sealed Deck" }
        e { return "Endor" }
        D { return "Death Star" }
        default { return "Unknown \[$set\]\\n" }
    }
}

proc showSide {alignment} {
    if {$alignment == "d"} {
        return "Dark"
    } else {
        return "Light"
    }
}

proc showInterSub {type} {
        switch $type {
          l { return "Lost Interrupt" }
          u { return "Used Interrupt" }
          o { return "Used Or Lost Interrupt" }
        }
        return ""
}

proc showEffectSub {type} {
        switch $type {
          e { return "Effect" }
          i { return "Immediate Effect" }
          m { return "Mobile Effect" }
          u { return "Utinni Effect" }
          t { return "Effect This Is" }
        }
        return ""
}

proc showLocSub {type} {
        switch $type {
          s { return "Site" }
          S { return "System" }
          e { return "Sector" }
        }
        return ""
}
proc showCharSub {type} {
    switch $type {
        a { return "Alien" }
        r { return "Rebel" }
        R { return "Rebel Alien" }
        d { return "Droid" }
        i { return "Imperial" }
        j { return "Jedi Master" }
    }
    return "UNKNOWN($type)"
}

proc showText {label text} {
    return "${label}: $text"
}

proc Window {args} {
global vTcl
    set cmd [lindex $args 0]
    set name [lindex $args 1]
    set newname [lindex $args 2]
    set rest [lrange $args 3 end]
    if {$name == "" || $cmd == ""} {return}
    if {$newname == ""} {
        set newname $name
    }
    set exists [winfo exists $newname]
    switch $cmd {
        show {
            if {$exists == "1" && $name != "."} {wm deiconify $name; return}
            if {[info procs vTclWindow(pre)$name] != ""} {
                eval "vTclWindow(pre)$name $newname $rest"
            }
            if {[info procs vTclWindow$name] != ""} {
                eval "vTclWindow$name $newname $rest"
            }
            if {[info procs vTclWindow(post)$name] != ""} {
                eval "vTclWindow(post)$name $newname $rest"
            }
        }
        hide    { if $exists {wm withdraw $newname; return} }
        iconify { if $exists {wm iconify $newname; return} }
        destroy { if $exists {destroy $newname; return} }
    }
}

#################################
# VTCL GENERATED GUI PROCEDURES
#

proc vTclWindow. {base} {
    if {$base == ""} {
        set base .
    }
    ###################
    # CREATING WIDGETS
    ###################
    wm focusmodel $base passive
    wm geometry $base 1x1+0+0
    wm maxsize $base 1137 834
    wm minsize $base 1 1
    wm overrideredirect $base 0
    wm resizable $base 1 1
    wm withdraw $base
    wm title $base "vt.tcl"
    ###################
    # SETTING GEOMETRY
    ###################
}

proc vTclWindow.top17 {base} {
    if {$base == ""} {
        set base .top17
    }
    if {[winfo exists $base]} {
        wm deiconify $base; return
    }
    ###################
    # CREATING WIDGETS
    ###################
    toplevel $base -class Toplevel
    wm focusmodel $base passive
    wm geometry $base 500x400
    wm maxsize $base 1137 834
    wm minsize $base 1 1
    wm overrideredirect $base 0
    wm resizable $base 1 1
    wm deiconify $base
    wm title $base "Windeck 2 CardTable"
    button $base.but20 \
        -command convertFile -padx 9 -pady 3 -text Convert 
    button $base.but21 \
        -command exit -padx 9 -pady 3 -text Quit 
    frame $base.cpd22 \
        -borderwidth 1 -height 30 -relief raised -width 30 
    scrollbar $base.cpd22.01 \
        -borderwidth 1 -command {.top17.cpd22.03 xview} -orient horiz \
        -width 10 
    scrollbar $base.cpd22.02 \
        -borderwidth 1 -command {.top17.cpd22.03 yview} -orient vert \
        -width 10 
    text $base.cpd22.03 \
        -font -Adobe-Courier-Medium-R-Normal-*-*-140-*-*-*-*-*-* -height 1 \
        -width 8 -xscrollcommand {.top17.cpd22.01 set} \
        -yscrollcommand {.top17.cpd22.02 set} 
    ###################
    # SETTING GEOMETRY
    ###################
    place $base.but20 \
        -x 0 -y 0 -anchor nw -bordermode ignore 
    place $base.but21 \
        -x 0 -relx 1 -y 0 -anchor ne -bordermode ignore 
    place $base.cpd22 \
        -x 0 -y 25 -width 0 -relwidth 1 -height -25 -relheight 1 -anchor nw 
    grid columnconf $base.cpd22 0 -weight 1
    grid rowconf $base.cpd22 0 -weight 1
    grid $base.cpd22.01 \
        -in .top17.cpd22 -column 0 -row 1 -columnspan 1 -rowspan 1 -sticky ew 
    grid $base.cpd22.02 \
        -in .top17.cpd22 -column 1 -row 0 -columnspan 1 -rowspan 1 -sticky ns 
    grid $base.cpd22.03 \
        -in .top17.cpd22 -column 0 -row 0 -columnspan 1 -rowspan 1 \
        -sticky nesw 
}

Window show .
Window show .top17

update
main $argc $argv
